### Keras FFN  ###

# function for creating features from lagged variables
lag_variables_to_features <- function(data, num_lags=1) {
  d <- embed(data[, -ncol(data)], num_lags+1) # this automatically drops NA, assumes target in last column
  d <- cbind(d, data[(num_lags+1):nrow(data), ncol(data)]) # add column for target, dropping num_lags
  return(d)
}

# test lagging function
set.seed(503)
dat <- replicate(3, rnorm(10, 0, 1))
dat

dat <- lag_variables_to_features(X, 2)
dat

## Load and process data
library(keras)
use_condaenv("r-tensorflow")

# load data - set up your own path and filename
path <- "C:/Users/Kris/Data/"
XY <- read.csv(paste0(path, 'keras_data_genEURUSD_L.csv'), header = F)

# visualize data
library(ggplot2)
library(reshape2)
df <- XY[1:240, -ncol(XY)]
df$index <- c(1:nrow(df))
df <- melt(df, id.vars = 'index')
ggplot(df, aes(index, value, color=variable)) +
  geom_line() +
  facet_wrap(~variable) +
  ggtitle("Scaled features (first ten days)")

pairs(XY[1:240, ], col='blue', main="Scatterplot of features and target (first ten days)")

# create lags
XY <- as.matrix(XY)
lags <- 7 
proc <- lag_variables_to_features(XY, lags)

# split into training, validation and test sets
train_length <- floor(0.5*nrow(proc))
val_length <- floor(0.25*nrow(proc))

X_train <- proc[1:train_length, -ncol(proc)]
Y_train_raw <- proc[1:train_length, ncol(proc)]
Y_train <- ifelse(Y_train_raw > 0, 1, 0)

X_val <- proc[(train_length+1):(train_length+val_length), -ncol(proc)]
Y_val_raw <- proc[(train_length+1):(train_length+val_length), ncol(proc)]
Y_val <- ifelse(Y_val_raw > 0, 1, 0)

X_test <- proc[(train_length+val_length+1):nrow(proc), -ncol(proc)]
Y_test_raw <- proc[(train_length+val_length+1):nrow(proc), ncol(proc)]
Y_test <- ifelse(Y_test_raw > 0, 1, 0)

# construct, compile and train model
model <- keras_model_sequential() 
model %>% 
  layer_dense(units = 150, activation = 'relu', input_shape = ncol(X_train)) %>%
  layer_dense(units = 150, activation = 'relu') %>%
  layer_dense(units = 150, activation = 'relu') %>%
  layer_dense(units = 1, activation = 'sigmoid') 

summary(model)

model %>% compile(
  loss = 'binary_crossentropy',
  optimizer = optimizer_rmsprop(lr=0.0001),
  metrics = c('accuracy')
)

# model checkpointing
filepath <- "C:/Users/Kris/Dropbox/RobotWealth/Research/DeepLearningForTrading/model.hdf5" # set up your own filepath
checkpoint <- callback_model_checkpoint(filepath = filepath, monitor = "val_acc", verbose = 1, 
                                        save_best_only = TRUE,
                                        save_weights_only = FALSE, mode = "auto")

history <- model %>% fit(
  X_train, Y_train,
  epochs = 75, batch_size = nrow(X_train),
  validation_data = list(X_val, Y_val), shuffle = TRUE,
  callbacks = list(checkpoint)
)

# plot training loss and accuracy
plot(history)
max(history$metrics$val_acc)

# load and evaluate best model
rm(model)
model <- keras:::keras$models$load_model(filepath)
model %>% evaluate(X_test, Y_test)

# get predictions on test set
preds <- model %>% predict_proba(X_test)
threshold <- 0.5
trades <- ifelse(preds >= threshold, Y_test_raw, ifelse(preds <= 1-threshold, -Y_test_raw, 0))
plot(cumsum(trades), type='l')